/*
//
// BEGIN SONGBIRD GPL
//
// This file is part of the Songbird web player.
//
// Copyright(c) 2005-2008 POTI, Inc.
// http://songbirdnest.com
//
// This file may be licensed under the terms of of the
// GNU General Public License Version 2 (the "GPL").
//
// Software distributed under the License is distributed
// on an "AS IS" basis, WITHOUT WARRANTY OF ANY KIND, either
// express or implied. See the GPL for the specific language
// governing rights and limitations.
//
// You should have received a copy of the GPL along with this
// program. If not, go to http://www.gnu.org/licenses/gpl.html
// or write to the Free Software Foundation, Inc.,
// 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
//
// END SONGBIRD GPL
//
*/

/**
 * \brief Some globally useful stuff for the local database library tests
 */

Components.utils.import("resource://app/jsmodules/sbProperties.jsm");
Components.utils.import("resource://app/jsmodules/ArrayConverter.jsm");

var gFilesToClose = [];
var gTailCallback = [];

function createNewLibrary(databaseGuid, databaseLocation) {

  var directory;
  if (databaseLocation) {
    directory = databaseLocation.QueryInterface(Ci.nsIFileURL).file;
  }
  else {
    directory = Cc["@mozilla.org/file/directory_service;1"].
                getService(Ci.nsIProperties).
                get("ProfD", Ci.nsIFile);
    directory.append("db");
  }

  var file = directory.clone();
  file.append(databaseGuid + ".db");

  var libraryFactory =
    Cc["@songbirdnest.com/Songbird/Library/LocalDatabase/LibraryFactory;1"]
      .getService(Ci.sbILibraryFactory);
  var hashBag = Cc["@mozilla.org/hash-property-bag;1"].
                createInstance(Ci.nsIWritablePropertyBag2);
  hashBag.setPropertyAsInterface("databaseFile", file);
  var library = libraryFactory.createLibrary(hashBag);
  try {
    library.clear();
  }
  catch(e) {
  }

  if (library) {
    var libraryManager = Cc["@songbirdnest.com/Songbird/library/Manager;1"].
                         getService(Ci.sbILibraryManager);
    libraryManager.registerLibrary(library, false);
  }
  return library;
}

function getPlatform() {
  var platform;
  try {
    var sysInfo =
      Components.classes["@mozilla.org/system-info;1"]
                .getService(Components.interfaces.nsIPropertyBag2);
    platform = sysInfo.getProperty("name");
  }
  catch (e) {
    dump("System-info not available, trying the user agent string.\n");
    var user_agent = navigator.userAgent;
    if (user_agent.indexOf("Windows") != -1)
      platform = "Windows_NT";
    else if (user_agent.indexOf("Mac OS X") != -1)
      platform = "Darwin";
    else if (user_agent.indexOf("Linux") != -1)
      platform = "Linux";
    else if (user_agent.indexOf("SunOS") != -1)
      platform = "SunOS";
  }
  return platform;
}

/**
 * Assert that all key/value pairs in mapA exist in mapB
 */
function assertObjectIsSubsetOf(mapA, mapB) {
  for (var id in mapA) {
    assertEqual(mapA[id], mapB[id]);
  }
}


/**
 * Dump information for an sbIJobProgress interface.
 * Used for debugging.
 */
function reportJobProgress(job, jobName) {
  log("\n\n\nMetadata - " + jobName + " job progress - " +
      job.progress + "/" + job.total + ", " + job.errorCount + " failed. " +
      "Status " + job.statusText + " (" + job.status + "), Title: " + job.titleText);
  var errorEnumerator = job.getErrorMessages();
  while (errorEnumerator.hasMore()) {
    log("MetadataJob " + jobName + " - failed url: " + errorEnumerator.getNext());
  }
  log("\n\n");
}


/**
 * Copy the given folder to tempName, returning an nsIFile
 * for the new location
 */
function getCopyOfFolder(folder, tempName) {
  assertNotEqual(folder, null);
  var tempFolder = getTempFolder();
  folder.copyTo(tempFolder, tempName);
  folder = tempFolder.clone();
  folder.append(tempName);
  assertEqual(folder.exists(), true);
  return folder;
}


/**
 * Copy the given folder to tempName, returning an nsIFile
 * for the new location
 */
function getCopyOfFile(file, tempName, optionalLocation) {
  assertNotEqual(file, null);
  var folder = optionalLocation ? optionalLocation : getTempFolder();
  file.copyTo(folder, tempName);
  file = folder.clone();
  file.append(tempName);
  assertEqual(file.exists(), true);
  return file;
}

/**
 * Get a temporary folder for use in metadata tests.
 * Will be removed in tail_metadatamanager.js
 */
var gTempFolder = null;
function getTempFolder() {
  if (gTempFolder) {
    return gTempFolder;
  }
  gTempFolder = Components.classes["@mozilla.org/file/directory_service;1"]
                       .getService(Components.interfaces.nsIProperties)
                       .get("TmpD", Components.interfaces.nsIFile);
  gTempFolder.append("songbird_metadata_tests.tmp");
  gTempFolder.createUnique(Components.interfaces.nsIFile.DIRECTORY_TYPE, 0777);
  return gTempFolder;
}


/**
 * Get rid of the temp folder created by getTempFolder.
 * Called in tail_metadatamanager.js
 */
function removeTempFolder() {
  if (gTempFolder && gTempFolder.exists()) {
    gTempFolder.remove(true);
  } else {
    log("\n\n\nMetadata Manager Test may not have performed cleanup.  Temp files may exist.\n\n\n");
  }
}
